==> web-ui-pack-1.0.2.tgz: package/dropdownElement.js <==
import WUPBaseElement from "./baseElement";
import WUPPopupElement from "./popup/popupElement";
import { WUPcssButton, WUPcssMenu } from "./styles";
const tagName = "wup-dropdown";
/** Dropdown element
 * @example
 * ```html
  <wup-dropdown class="custom">
    <button type="button">Click Me</button>
    <wup-popup w-placement="bottom-middle" w-animation="stack">
      <ul>
        <li><button type="button">A</button></li>
        <li><button type="button">B</button></li>
        <li><button type="button">C</button></li>
      </ul>
    </wup-popup>
  </wup-dropdown>
 * ```
 * @tutorial Troubleshooting
 * * in comparison with Combobox Dropdown doesn't contain input but can contain in popup any items (like nav-menu etc.) */
export default class WUPDropdownElement extends WUPBaseElement {
    #ctr = this.constructor;
    static get $style() {
        return `${super.$style}
      :host {
        contain: style;
        display: inline-block;
      }${WUPcssButton(":host button")}
      :host button {
        min-width: initial;
        margin: 0;
        padding: 0.7em;
      }${WUPcssMenu(":host>[menu]")}`;
    }
    /** Default options applied to every element. Change it to configure default behavior
     * * @tutorial Troubleshooting
     * * Popup-related options are not observed so to change it use `WUPDropdownElement.$defaults` or `element.$refPopup.$options` directly */
    static $defaults = {
        ...WUPPopupElement.$defaults,
        animation: 1 /* PopupAnimations.drawer */,
        openCase: 8 /* PopupOpenCases.onClick */ | 4 /* PopupOpenCases.onFocus */,
        closeOnPopupClick: true,
        minHeightByTarget: true,
        minWidthByTarget: true,
        placement: [
            WUPPopupElement.$placements.$bottom.$start,
            WUPPopupElement.$placements.$bottom.$end,
            WUPPopupElement.$placements.$top.$start,
            WUPPopupElement.$placements.$top.$end,
            WUPPopupElement.$placements.$bottom.$start.$resizeHeight,
            WUPPopupElement.$placements.$bottom.$end.$resizeHeight,
            WUPPopupElement.$placements.$top.$start.$resizeHeight,
            WUPPopupElement.$placements.$top.$end.$resizeHeight,
        ],
    };
    /** Reference to nested HTMLElement tied with $options.label */
    $refTitle = this.firstElementChild;
    /** Reference to popupMenu */
    $refPopup = this.lastElementChild;
    /** Reference to list */
    $refMenu = this.lastElementChild;
    gotReady() {
        this.$refTitle = this.firstElementChild;
        this.$refPopup = this.lastElementChild;
        if (this.$refTitle === this.$refPopup || !this.$refPopup.$open) {
            this.throwError("Invalid structure. Expected 1st element: <any/>, last element: <wup-popup/>");
        }
        else {
            this.$refPopup.setAttribute("menu", "");
            this.$refPopup.goOpen = this.goOpenPopup.bind(this);
            this.$refPopup.goClose = this.goClosePopup.bind(this);
            // find popup props from attributes
            const excluded = new Set(); // popup props that assigned from attributes
            const proto = Object.getPrototypeOf(this.$refPopup).constructor;
            const mappedAttr = proto.mappedAttributes;
            this.$refPopup.getAttributeNames().forEach((a) => {
                a = a.startsWith("w-") ? a.substring(2) : a;
                const m = mappedAttr[a];
                m && excluded.add(m.prop ?? a);
            });
            Object.keys(this._opts).forEach((k) => {
                if (!excluded.has(k)) {
                    // @ts-expect-error - because closeOnPopupClick doesn't exist on popup
                    this.$refPopup.$options[k] = this._opts[k];
                }
            });
            // WA
            const menu = this.$refPopup.querySelector("ul,ol,[items]") || this.$refPopup;
            menu.id = menu.id || this.#ctr.$uniqueId;
            this.$refMenu = menu;
            const lbl = this.$refTitle;
            lbl.setAttribute("aria-owns", menu.id);
            lbl.setAttribute("aria-controls", menu.id);
            lbl.setAttribute("aria-haspopup", "listbox");
            lbl.setAttribute("aria-expanded", false);
            // !menu.hasAttribute("role") && menu.setAttribute("role", "listbox");
            // li.setAttribute("role", "option");
            !menu.hasAttribute("tabindex") && menu.setAttribute("tabindex", -1); // otherwise Firefox move focus into it by keydown 'Tab'
            // lbl.id = lbl.id || this.#ctr.$uniqueId;
            // menu.setAttribute("aria-labelledby", lbl.id);
        }
        super.gotReady();
    }
    /** Custom function to override default `WUPPopupElement.prototype.goClose` */
    goOpenPopup(openCase, ev) {
        const p = WUPPopupElement.prototype.goOpen.call(this.$refPopup, openCase, ev);
        const t = this.$refPopup.$options.target;
        t.setAttribute("aria-expanded", true); // NiceToHave: move it to popup side after refactoring ???
        t.style.zIndex = `${+getComputedStyle(this.$refPopup).zIndex + 2}`; // inc z-index for btn to allow animation-stack works properly
        return p;
    }
    /** Custom function to override default `WUPPopupElement.prototype.goClose` */
    goClosePopup(closeCase, ev) {
        if (closeCase === 4 /* PopupCloseCases.onPopupClick */ && !this._opts.closeOnPopupClick) {
            return Promise.resolve(false);
        }
        const p = WUPPopupElement.prototype.goClose.call(this.$refPopup, closeCase, ev).finally(() => {
            t.style.zIndex = "";
            this.removeEmptyStyle.call(t);
        });
        const t = this.$refPopup.$options.target;
        t.setAttribute("aria-expanded", false);
        return p;
    }
}
customElements.define(tagName, WUPDropdownElement);
// todo issue: animation:stack-right. If target is partially hidden in scrollable parent popup changes Y to be full visible but it's not ok; In this case popup must be also partially visible and listen only target

==> web-ui-pack-1.2.4.tgz: package/dropdownElement.js <==
import WUPBaseElement from "./baseElement";
import WUPPopupElement from "./popup/popupElement";
import { WUPcssButton, WUPcssMenu } from "./styles";
const tagName = "wup-dropdown";
export default class WUPDropdownElement extends WUPBaseElement {
    #ctr = this.constructor;
    static get $style() {
        return `${super.$style}
      :host {
        contain: style;
        display: inline-block;
      }${WUPcssButton(":host button")}
      :host button {
        min-width: initial;
        margin: 0;
        padding: 0.7em;
      }${WUPcssMenu(":host>[menu]")}`;
    }
    static $defaults = {
        ...WUPPopupElement.$defaults,
        animation: 1,
        openCase: 8 | 4,
        closeOnPopupClick: true,
        minHeightByTarget: true,
        minWidthByTarget: true,
        placement: [
            WUPPopupElement.$placements.$bottom.$start,
            WUPPopupElement.$placements.$bottom.$end,
            WUPPopupElement.$placements.$top.$start,
            WUPPopupElement.$placements.$top.$end,
            WUPPopupElement.$placements.$bottom.$start.$resizeHeight,
            WUPPopupElement.$placements.$bottom.$end.$resizeHeight,
            WUPPopupElement.$placements.$top.$start.$resizeHeight,
            WUPPopupElement.$placements.$top.$end.$resizeHeight,
        ],
    };
    $refTitle = this.firstElementChild;
    $refPopup = this.lastElementChild;
    $refMenu = this.lastElementChild;
    gotReady() {
        this.$refTitle = this.firstElementChild;
        this.$refPopup = this.lastElementChild;
        if (this.$refTitle === this.$refPopup || !this.$refPopup.$open) {
            this.throwError("Invalid structure. Expected 1st element: <any/>, last element: <wup-popup/>");
        }
        else {
            this.$refPopup.setAttribute("menu", "");
            this.$refPopup.goOpen = this.goOpenPopup.bind(this);
            this.$refPopup.goClose = this.goClosePopup.bind(this);
            const excluded = new Set();
            const proto = Object.getPrototypeOf(this.$refPopup).constructor;
            const mappedAttr = proto.mappedAttributes;
            this.$refPopup.getAttributeNames().forEach((a) => {
                a = a.startsWith("w-") ? a.substring(2) : a;
                const m = mappedAttr[a];
                m && excluded.add(m.prop ?? a);
            });
            Object.keys(this._opts).forEach((k) => {
                if (!excluded.has(k)) {
                    this.$refPopup.$options[k] = this._opts[k];
                }
            });
            const menu = this.$refPopup.querySelector("ul,ol,[items]") || this.$refPopup;
            menu.id = menu.id || this.#ctr.$uniqueId;
            this.$refMenu = menu;
            const lbl = this.$refTitle;
            lbl.setAttribute("aria-owns", menu.id);
            lbl.setAttribute("aria-controls", menu.id);
            lbl.setAttribute("aria-haspopup", "listbox");
            lbl.setAttribute("aria-expanded", false);
            !menu.hasAttribute("tabindex") && menu.setAttribute("tabindex", -1);
        }
        super.gotReady();
    }
    goOpenPopup(openCase, ev) {
        const p = WUPPopupElement.prototype.goOpen.call(this.$refPopup, openCase, ev);
        const t = this.$refPopup.$options.target;
        t.setAttribute("aria-expanded", true);
        t.style.zIndex = `${+getComputedStyle(this.$refPopup).zIndex + 2}`;
        return p;
    }
    goClosePopup(closeCase, ev) {
        if (closeCase === 4 && !this._opts.closeOnPopupClick) {
            return Promise.resolve(false);
        }
        const p = WUPPopupElement.prototype.goClose.call(this.$refPopup, closeCase, ev).finally(() => {
            t.style.zIndex = "";
            this.removeEmptyStyle.call(t);
        });
        const t = this.$refPopup.$options.target;
        t.setAttribute("aria-expanded", false);
        return p;
    }
}
customElements.define(tagName, WUPDropdownElement);
